@@ -24,14 +24,18 @@ from aiohomekit.crypto.chacha20poly1305 import (
 )
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
+    AccessoryNotFoundError,
     AuthenticationError,
     ConnectionError,
     HomeKitException,
+    HttpErrorResponse,
+    TimeoutError,
 )
 from aiohomekit.http import HttpContentTypes
 from aiohomekit.http.response import HttpResponse
 from aiohomekit.protocol import get_session_keys
 from aiohomekit.protocol.tlv import TLV
+from aiohomekit.zeroconf import async_find_device_ip_and_port
 
 logger = logging.getLogger(__name__)
 
@@ -69,7 +73,7 @@ class InsecureHomeKitProtocol(asyncio.Protocol):
         self.result_cbs.append(result)
 
         try:
-            return await asyncio.wait_for(result, 10)
+            return await asyncio.wait_for(result, 30)
         except asyncio.TimeoutError:
             self.transport.write_eof()
             self.transport.close()
@@ -223,7 +227,7 @@ class HomeKitConnection:
             return
         self.closing = False
         self.start_connector()
-        await self._connector
+        await asyncio.shield(self._connector)
 
     async def stop_connector(self):
         if not self._connector:
@@ -258,7 +262,9 @@ class HomeKitConnection:
 
     async def put_json(self, target, body):
         response = await self.put(
-            target, json.dumps(body).encode("utf-8"), content_type=HttpContentTypes.TLV,
+            target,
+            json.dumps(body).encode("utf-8"),
+            content_type=HttpContentTypes.JSON,
         )
 
         if response.code == 204:
@@ -320,9 +326,13 @@ class HomeKitConnection:
         return parsed
 
     async def post_tlv(self, target, body, expected=None):
-        response = await self.post(
-            target, TLV.encode_list(body), content_type=HttpContentTypes.TLV,
-        )
+        try:
+            response = await self.post(
+                target, TLV.encode_list(body), content_type=HttpContentTypes.TLV,
+            )
+        except HttpErrorResponse as e:
+            self.transport.close()
+            response = e.response
         body = TLV.decode_bytes(response.body, expected=expected)
         return body
 
@@ -338,6 +348,11 @@ class HomeKitConnection:
         :param headers: a list of (header, value) tuples (optional)
         :param body: The body of the request (optional)
         """
+        if not self.protocol:
+            raise AccessoryDisconnectedError(
+                "Connection lost before request could be sent"
+            )
+
         buffer = []
         buffer.append(
             "{method} {target} HTTP/1.1".format(method=method.upper(), target=target,)
@@ -367,7 +382,19 @@ class HomeKitConnection:
         # https://github.com/jlusiardi/homekit_python/issues/12
         # https://github.com/jlusiardi/homekit_python/issues/16
 
-        return await self.protocol.send_bytes(request_bytes)
+        logger.debug("%s: raw request: %r", self.host, request_bytes)
+        resp = await self.protocol.send_bytes(request_bytes)
+
+        if resp.code >= 400 and resp.code <= 499:
+            logger.debug(f"Got HTTP error {resp.code} for {method} against {target}")
+            raise HttpErrorResponse(
+                f"Got HTTP error {resp.code} for {method} against {target}",
+                response=resp,
+            )
+
+        logger.debug("%s: raw response: %r", self.host, resp.body)
+
+        return resp
 
     async def close(self):
         """
@@ -402,10 +429,19 @@ class HomeKitConnection:
     async def _connect_once(self):
         loop = asyncio.get_event_loop()
 
+        logger.debug("Attempting connection to %s:%s", self.host, self.port)
+
         try:
-            self.transport, self.protocol = await loop.create_connection(
-                lambda: InsecureHomeKitProtocol(self), self.host, self.port
+            self.transport, self.protocol = await asyncio.wait_for(
+                loop.create_connection(
+                    lambda: InsecureHomeKitProtocol(self), self.host, self.port
+                ),
+                timeout=10,
             )
+
+        except asyncio.TimeoutError:
+            raise TimeoutError("Timeout")
+
         except OSError as e:
             raise ConnectionError(str(e))
 
@@ -417,32 +453,31 @@ class HomeKitConnection:
         # There is aiozeroconf but that doesn't work on Windows until python 3.9
         # In HASS, zeroconf is a service provided by HASS itself and want to be able to
         # leverage that instead.
+        interval = 0.5
+
         while not self.closing:
             try:
-                await self._connect_once()
+                return await self._connect_once()
 
             except AuthenticationError:
                 # Authentication errors should bubble up because auto-reconnect is unlikely to help
                 raise
 
+            except asyncio.CancelledError:
+                return
+
             except HomeKitException:
-                interval = self._retry_interval = min(60, 1.5 * self._retry_interval)
                 logger.debug(
                     "Connecting to accessory failed. Retrying in %i seconds", interval
                 )
-                await asyncio.sleep(interval)
-                continue
 
             except Exception:
                 logger.exception(
                     "Unexpected error whilst trying to connect to accessory. Will retry."
                 )
-                interval = self._retry_interval = min(60, 1.5 * self._retry_interval)
-                await asyncio.sleep(interval)
-                continue
 
-            self._retry_interval = 0.5
-            return
+            interval = min(60, 1.5 * interval)
+            await asyncio.sleep(interval)
 
     def event_received(self, event):
         if not self.owner:
@@ -479,6 +514,13 @@ class SecureHomeKitConnection(HomeKitConnection):
     async def _connect_once(self):
         self.is_secure = False
 
+        try:
+            self.host, self.port = await async_find_device_ip_and_port(
+                self.pairing_data["AccessoryPairingID"]
+            )
+        except AccessoryNotFoundError:
+            pass
+
         await super()._connect_once()
 
         state_machine = get_session_keys(self.pairing_data)
