@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING, Iterable, Optional
 
 from aiohomekit.model import ToDictMixin
 from aiohomekit.model.characteristics import Characteristic, CharacteristicsTypes
@@ -25,6 +25,29 @@ if TYPE_CHECKING:
     from aiohomekit.model import Accessory
 
 
+class Characteristics:
+    def __init__(self):
+        self._characteristics = []
+
+    def append(self, char):
+        self._characteristics.append(char)
+
+    def __iter__(self):
+        return iter(self._characteristics)
+
+    def filter(self, char_types=None) -> Iterable[Characteristic]:
+        matches = iter(self)
+
+        if char_types:
+            char_types = [CharacteristicsTypes.get_uuid(c) for c in char_types]
+            matches = filter(lambda char: char.type in char_types, matches)
+
+        return matches
+
+    def first(self, char_types=None) -> Characteristic:
+        return next(self.filter(char_types=char_types))
+
+
 class Service(ToDictMixin):
     def __init__(
         self,
@@ -40,7 +63,7 @@ class Service(ToDictMixin):
 
         self.accessory = accessory
         self.iid = accessory.get_next_id()
-        self.characteristics = []
+        self.characteristics = Characteristics()
         self.characteristics_by_type = {}
         self.linked = []
 
@@ -94,6 +117,19 @@ class Service(ToDictMixin):
     def add_linked_service(self, service: "Service"):
         self.linked.append(service)
 
+    def build_update(self, payload):
+        """
+        Given a payload in the form of {CHAR_TYPE: value}, render in a form suitable to pass
+        to put_characteristics using aid and iid.
+        """
+        result = []
+
+        for char_type, value in payload.items():
+            char = self[char_type]
+            result.append((self.accessory.aid, char.iid, value))
+
+        return result
+
     def to_accessory_and_service_list(self):
         characteristics_list = []
         for c in self.characteristics:
